# Generated by roxygen2: do not edit by hand

S3method(knit_print,Backfill)
S3method(knit_print,CohortSizeConst)
S3method(knit_print,CohortSizeDLT)
S3method(knit_print,CohortSizeMax)
S3method(knit_print,CohortSizeMin)
S3method(knit_print,CohortSizeOrdinal)
S3method(knit_print,CohortSizeParts)
S3method(knit_print,CohortSizeRandom)
S3method(knit_print,CohortSizeRange)
S3method(knit_print,DADesign)
S3method(knit_print,DataParts)
S3method(knit_print,Design)
S3method(knit_print,DesignGrouped)
S3method(knit_print,DesignOrdinal)
S3method(knit_print,DualDesign)
S3method(knit_print,DualEndpoint)
S3method(knit_print,DualResponsesDesign)
S3method(knit_print,DualResponsesSamplesDesign)
S3method(knit_print,Effloglog)
S3method(knit_print,GeneralData)
S3method(knit_print,GeneralModel)
S3method(knit_print,IncrementsDoseLevels)
S3method(knit_print,IncrementsHSRBeta)
S3method(knit_print,IncrementsMin)
S3method(knit_print,IncrementsOrdinal)
S3method(knit_print,IncrementsRelative)
S3method(knit_print,IncrementsRelativeDLT)
S3method(knit_print,IncrementsRelativeDLTCurrent)
S3method(knit_print,IncrementsRelativeParts)
S3method(knit_print,LogisticIndepBeta)
S3method(knit_print,LogisticKadane)
S3method(knit_print,LogisticKadaneBetaGamma)
S3method(knit_print,LogisticLogNormal)
S3method(knit_print,LogisticLogNormalGrouped)
S3method(knit_print,LogisticLogNormalMixture)
S3method(knit_print,LogisticLogNormalOrdinal)
S3method(knit_print,LogisticLogNormalSub)
S3method(knit_print,LogisticNormalFixedMixture)
S3method(knit_print,LogisticNormalMixture)
S3method(knit_print,ModelParamsNormal)
S3method(knit_print,NextBestDualEndpoint)
S3method(knit_print,NextBestInfTheory)
S3method(knit_print,NextBestMTD)
S3method(knit_print,NextBestMaxGain)
S3method(knit_print,NextBestMaxGainSamples)
S3method(knit_print,NextBestMinDist)
S3method(knit_print,NextBestNCRM)
S3method(knit_print,NextBestNCRMLoss)
S3method(knit_print,NextBestOrdinal)
S3method(knit_print,NextBestProbMTDLTE)
S3method(knit_print,NextBestProbMTDMinDist)
S3method(knit_print,NextBestTD)
S3method(knit_print,NextBestTDsamples)
S3method(knit_print,NextBestThreePlusThree)
S3method(knit_print,OneParExpPrior)
S3method(knit_print,OneParLogNormalPrior)
S3method(knit_print,OpeningAll)
S3method(knit_print,OpeningAny)
S3method(knit_print,OpeningList)
S3method(knit_print,OpeningMinCohorts)
S3method(knit_print,OpeningMinDose)
S3method(knit_print,OpeningMinResponses)
S3method(knit_print,OpeningNone)
S3method(knit_print,RecruitmentRatio)
S3method(knit_print,RecruitmentUnlimited)
S3method(knit_print,RuleDesign)
S3method(knit_print,RuleDesignOrdinal)
S3method(knit_print,SafetyWindow)
S3method(knit_print,SafetyWindowConst)
S3method(knit_print,SafetyWindowSize)
S3method(knit_print,StartingDose)
S3method(knit_print,StoppingAll)
S3method(knit_print,StoppingAny)
S3method(knit_print,StoppingCohortsNearDose)
S3method(knit_print,StoppingHighestDose)
S3method(knit_print,StoppingList)
S3method(knit_print,StoppingLowestDoseHSRBeta)
S3method(knit_print,StoppingMTDCV)
S3method(knit_print,StoppingMTDdistribution)
S3method(knit_print,StoppingMaxGainCIRatio)
S3method(knit_print,StoppingMinCohorts)
S3method(knit_print,StoppingMinPatients)
S3method(knit_print,StoppingMissingDose)
S3method(knit_print,StoppingOrdinal)
S3method(knit_print,StoppingPatientsNearDose)
S3method(knit_print,StoppingSpecificDose)
S3method(knit_print,StoppingTDCIRatio)
S3method(knit_print,StoppingTargetBiomarker)
S3method(knit_print,StoppingTargetProb)
S3method(knit_print,TDDesign)
S3method(knit_print,TDsamplesDesign)
S3method(plot,gtable)
S3method(print,gtable)
export("%>%")
export(.Backfill)
export(.CohortSizeConst)
export(.CohortSizeDLT)
export(.CohortSizeMax)
export(.CohortSizeMin)
export(.CohortSizeOrdinal)
export(.CohortSizeParts)
export(.CohortSizeRandom)
export(.CohortSizeRange)
export(.CrmPackClass)
export(.DADesign)
export(.DALogisticLogNormal)
export(.DASimulations)
export(.Data)
export(.DataDA)
export(.DataDual)
export(.DataGrouped)
export(.DataMixture)
export(.DataOrdinal)
export(.DataParts)
export(.DefaultBackfill)
export(.DefaultCohortSize)
export(.DefaultCohortSizeConst)
export(.DefaultCohortSizeDLT)
export(.DefaultCohortSizeMax)
export(.DefaultCohortSizeMin)
export(.DefaultCohortSizeOrdinal)
export(.DefaultCohortSizeParts)
export(.DefaultCohortSizeRandom)
export(.DefaultCohortSizeRange)
export(.DefaultDADesign)
export(.DefaultDALogisticLogNormal)
export(.DefaultDASimulations)
export(.DefaultData)
export(.DefaultDataDA)
export(.DefaultDataDual)
export(.DefaultDataGeneral)
export(.DefaultDataGrouped)
export(.DefaultDataMixture)
export(.DefaultDataOrdinal)
export(.DefaultDataParts)
export(.DefaultDesign)
export(.DefaultDesignGrouped)
export(.DefaultDesignOrdinal)
export(.DefaultDualDesign)
export(.DefaultDualEndpoint)
export(.DefaultDualEndpointBeta)
export(.DefaultDualEndpointEmax)
export(.DefaultDualEndpointRW)
export(.DefaultDualResponsesDesign)
export(.DefaultDualResponsesSamplesDesign)
export(.DefaultDualSimulations)
export(.DefaultDualSimulationsSummary)
export(.DefaultEffFlexi)
export(.DefaultEffloglog)
export(.DefaultFractionalCRM)
export(.DefaultGeneralModel)
export(.DefaultGeneralSimulations)
export(.DefaultGeneralSimulationsSummary)
export(.DefaultIncrements)
export(.DefaultIncrementsDoseLevels)
export(.DefaultIncrementsHSRBeta)
export(.DefaultIncrementsMaxToxProb)
export(.DefaultIncrementsMin)
export(.DefaultIncrementsOrdinal)
export(.DefaultIncrementsRelative)
export(.DefaultIncrementsRelativeDLT)
export(.DefaultIncrementsRelativeDLTCurrent)
export(.DefaultIncrementsRelativeParts)
export(.DefaultLogisticIndepBeta)
export(.DefaultLogisticKadane)
export(.DefaultLogisticKadaneBetaGamma)
export(.DefaultLogisticLogNormal)
export(.DefaultLogisticLogNormalGrouped)
export(.DefaultLogisticLogNormalMixture)
export(.DefaultLogisticLogNormalOrdinal)
export(.DefaultLogisticLogNormalSub)
export(.DefaultLogisticNormal)
export(.DefaultLogisticNormalFixedMixture)
export(.DefaultLogisticNormalMixture)
export(.DefaultMcmcOptions)
export(.DefaultModelEff)
export(.DefaultModelLogNormal)
export(.DefaultModelParamsNormal)
export(.DefaultModelPseudo)
export(.DefaultModelTox)
export(.DefaultNextBest)
export(.DefaultNextBestDualEndpoint)
export(.DefaultNextBestEWOC)
export(.DefaultNextBestInfTheory)
export(.DefaultNextBestMTD)
export(.DefaultNextBestMaxGain)
export(.DefaultNextBestMaxGainSamples)
export(.DefaultNextBestMinDist)
export(.DefaultNextBestNCRM)
export(.DefaultNextBestNCRMLoss)
export(.DefaultNextBestOrdinal)
export(.DefaultNextBestProbMTDLTE)
export(.DefaultNextBestProbMTDMinDist)
export(.DefaultNextBestTD)
export(.DefaultNextBestTDsamples)
export(.DefaultNextBestThreePlusThree)
export(.DefaultOneParExpPrior)
export(.DefaultOneParLogNormalPrior)
export(.DefaultOpening)
export(.DefaultOpeningAll)
export(.DefaultOpeningAny)
export(.DefaultOpeningList)
export(.DefaultOpeningMinCohorts)
export(.DefaultOpeningMinDose)
export(.DefaultOpeningMinResponses)
export(.DefaultOpeningNone)
export(.DefaultProbitLogNormal)
export(.DefaultProbitLogNormalRel)
export(.DefaultPseudoDualFlexiSimulations)
export(.DefaultPseudoDualSimulations)
export(.DefaultPseudoDualSimulationsSummary)
export(.DefaultPseudoSimulations)
export(.DefaultPseudoSimulationsSummary)
export(.DefaultRecruitment)
export(.DefaultRecruitmentRatio)
export(.DefaultRecruitmentUnlimited)
export(.DefaultRuleDesign)
export(.DefaultRuleDesignOrdinal)
export(.DefaultSafetyWindow)
export(.DefaultSafetyWindowConst)
export(.DefaultSafetyWindowSize)
export(.DefaultSamples)
export(.DefaultSimulations)
export(.DefaultSimulationsSummary)
export(.DefaultStoppingAll)
export(.DefaultStoppingAny)
export(.DefaultStoppingCohortsNearDose)
export(.DefaultStoppingExternal)
export(.DefaultStoppingHighestDose)
export(.DefaultStoppingList)
export(.DefaultStoppingLowestDoseHSRBeta)
export(.DefaultStoppingMTDCV)
export(.DefaultStoppingMTDdistribution)
export(.DefaultStoppingMaxGainCIRatio)
export(.DefaultStoppingMinCohorts)
export(.DefaultStoppingMinPatients)
export(.DefaultStoppingMissingDose)
export(.DefaultStoppingOrdinal)
export(.DefaultStoppingPatientsNearDose)
export(.DefaultStoppingSpecificDose)
export(.DefaultStoppingTDCIRatio)
export(.DefaultStoppingTargetBiomarker)
export(.DefaultStoppingTargetProb)
export(.DefaultTDDesign)
export(.DefaultTDsamplesDesign)
export(.DefaultTITELogisticLogNormal)
export(.Design)
export(.DesignGrouped)
export(.DesignOrdinal)
export(.DualDesign)
export(.DualEndpoint)
export(.DualEndpointBeta)
export(.DualEndpointEmax)
export(.DualEndpointRW)
export(.DualResponsesDesign)
export(.DualResponsesSamplesDesign)
export(.DualSimulations)
export(.DualSimulationsSummary)
export(.EffFlexi)
export(.Effloglog)
export(.FractionalCRM)
export(.GeneralData)
export(.GeneralModel)
export(.GeneralSimulations)
export(.GeneralSimulationsSummary)
export(.IncrementsDoseLevels)
export(.IncrementsHSRBeta)
export(.IncrementsMaxToxProb)
export(.IncrementsMin)
export(.IncrementsOrdinal)
export(.IncrementsRelative)
export(.IncrementsRelativeDLT)
export(.IncrementsRelativeDLTCurrent)
export(.IncrementsRelativeParts)
export(.LogisticIndepBeta)
export(.LogisticKadane)
export(.LogisticKadaneBetaGamma)
export(.LogisticLogNormal)
export(.LogisticLogNormalGrouped)
export(.LogisticLogNormalMixture)
export(.LogisticLogNormalOrdinal)
export(.LogisticLogNormalSub)
export(.LogisticNormal)
export(.LogisticNormalFixedMixture)
export(.LogisticNormalMixture)
export(.McmcOptions)
export(.ModelEff)
export(.ModelLogNormal)
export(.ModelParamsNormal)
export(.ModelPseudo)
export(.ModelTox)
export(.NextBestDualEndpoint)
export(.NextBestEWOC)
export(.NextBestInfTheory)
export(.NextBestMTD)
export(.NextBestMaxGain)
export(.NextBestMaxGainSamples)
export(.NextBestMinDist)
export(.NextBestNCRM)
export(.NextBestNCRMLoss)
export(.NextBestOrdinal)
export(.NextBestProbMTDLTE)
export(.NextBestProbMTDMinDist)
export(.NextBestTD)
export(.NextBestTDsamples)
export(.NextBestThreePlusThree)
export(.OneParExpPrior)
export(.OneParLogNormalPrior)
export(.Opening)
export(.OpeningAll)
export(.OpeningAny)
export(.OpeningList)
export(.OpeningMinCohorts)
export(.OpeningMinDose)
export(.OpeningMinResponses)
export(.OpeningNone)
export(.ProbitLogNormal)
export(.ProbitLogNormalRel)
export(.PseudoDualFlexiSimulations)
export(.PseudoDualSimulations)
export(.PseudoDualSimulationsSummary)
export(.PseudoSimulations)
export(.PseudoSimulationsSummary)
export(.Recruitment)
export(.RecruitmentRatio)
export(.RecruitmentUnlimited)
export(.RuleDesign)
export(.RuleDesignOrdinal)
export(.SafetyWindowConst)
export(.SafetyWindowSize)
export(.Samples)
export(.Simulations)
export(.SimulationsSummary)
export(.StoppingAll)
export(.StoppingAny)
export(.StoppingCohortsNearDose)
export(.StoppingExternal)
export(.StoppingHighestDose)
export(.StoppingList)
export(.StoppingLowestDoseHSRBeta)
export(.StoppingMTDCV)
export(.StoppingMTDdistribution)
export(.StoppingMaxGainCIRatio)
export(.StoppingMinCohorts)
export(.StoppingMinPatients)
export(.StoppingMissingDose)
export(.StoppingOrdinal)
export(.StoppingPatientsNearDose)
export(.StoppingSpecificDose)
export(.StoppingTDCIRatio)
export(.StoppingTargetBiomarker)
export(.StoppingTargetProb)
export(.TDDesign)
export(.TDsamplesDesign)
export(.TITELogisticLogNormal)
export(Backfill)
export(CohortSizeConst)
export(CohortSizeDLT)
export(CohortSizeMax)
export(CohortSizeMin)
export(CohortSizeOrdinal)
export(CohortSizeParts)
export(CohortSizeRandom)
export(CohortSizeRange)
export(DADesign)
export(DALogisticLogNormal)
export(DASimulations)
export(Data)
export(DataDA)
export(DataDual)
export(DataGrouped)
export(DataMixture)
export(DataOrdinal)
export(DataParts)
export(Design)
export(DesignGrouped)
export(DesignOrdinal)
export(DualDesign)
export(DualEndpoint)
export(DualEndpointBeta)
export(DualEndpointEmax)
export(DualEndpointRW)
export(DualResponsesDesign)
export(DualResponsesSamplesDesign)
export(DualSimulations)
export(EffFlexi)
export(Effloglog)
export(FractionalCRM)
export(GeneralSimulations)
export(IncrementsDoseLevels)
export(IncrementsHSRBeta)
export(IncrementsMaxToxProb)
export(IncrementsMin)
export(IncrementsOrdinal)
export(IncrementsRelative)
export(IncrementsRelativeDLT)
export(IncrementsRelativeDLTCurrent)
export(IncrementsRelativeParts)
export(LogisticIndepBeta)
export(LogisticKadane)
export(LogisticKadaneBetaGamma)
export(LogisticLogNormal)
export(LogisticLogNormalGrouped)
export(LogisticLogNormalMixture)
export(LogisticLogNormalOrdinal)
export(LogisticLogNormalSub)
export(LogisticNormal)
export(LogisticNormalFixedMixture)
export(LogisticNormalMixture)
export(McmcOptions)
export(MinimalInformative)
export(ModelLogNormal)
export(ModelParamsNormal)
export(NextBestDualEndpoint)
export(NextBestEWOC)
export(NextBestInfTheory)
export(NextBestMTD)
export(NextBestMaxGain)
export(NextBestMaxGainSamples)
export(NextBestMinDist)
export(NextBestNCRM)
export(NextBestNCRMLoss)
export(NextBestOrdinal)
export(NextBestProbMTDLTE)
export(NextBestProbMTDMinDist)
export(NextBestTD)
export(NextBestTDsamples)
export(NextBestThreePlusThree)
export(OneParExpPrior)
export(OneParLogNormalPrior)
export(OpeningAll)
export(OpeningAny)
export(OpeningList)
export(OpeningMinCohorts)
export(OpeningMinDose)
export(OpeningMinResponses)
export(OpeningNone)
export(ProbitLogNormal)
export(ProbitLogNormalRel)
export(PseudoDualFlexiSimulations)
export(PseudoDualSimulations)
export(PseudoSimulations)
export(Quantiles2LogisticNormal)
export(RecruitmentRatio)
export(RecruitmentUnlimited)
export(RuleDesign)
export(RuleDesignOrdinal)
export(SafetyWindowConst)
export(SafetyWindowSize)
export(Samples)
export(Simulations)
export(StoppingAll)
export(StoppingAny)
export(StoppingCohortsNearDose)
export(StoppingExternal)
export(StoppingHighestDose)
export(StoppingList)
export(StoppingLowestDoseHSRBeta)
export(StoppingMTDCV)
export(StoppingMTDdistribution)
export(StoppingMaxGainCIRatio)
export(StoppingMinCohorts)
export(StoppingMinPatients)
export(StoppingMissingDose)
export(StoppingOrdinal)
export(StoppingPatientsNearDose)
export(StoppingSpecificDose)
export(StoppingTDCIRatio)
export(StoppingTargetBiomarker)
export(StoppingTargetProb)
export(TDDesign)
export(TDsamplesDesign)
export(TITELogisticLogNormal)
export(ThreePlusThreeDesign)
export(approximate)
export(assert_equal)
export(assert_format)
export(assert_length)
export(assert_probabilities)
export(assert_probability)
export(assert_probability_range)
export(assert_range)
export(biomarker)
export(check_equal)
export(check_format)
export(check_length)
export(check_probabilities)
export(check_probability)
export(check_probability_range)
export(check_range)
export(crmPackExample)
export(crmPackHelp)
export(dapply)
export(disable_logging)
export(dose)
export(doseFunction)
export(dose_grid_range)
export(efficacy)
export(efficacyFunction)
export(enable_logging)
export(examine)
export(expect_format)
export(expect_probabilities)
export(expect_probability)
export(expect_probability_range)
export(expect_range)
export(fit)
export(fitGain)
export(fitPEM)
export(gain)
export(getEff)
export(h_all_equivalent)
export(h_check_fun_formals)
export(h_convert_ordinal_data)
export(h_convert_ordinal_model)
export(h_convert_ordinal_samples)
export(h_default_if_empty)
export(h_find_interval)
export(h_format_number)
export(h_in_range)
export(h_info_theory_dist)
export(h_is_positive_definite)
export(h_jags_add_dummy)
export(h_jags_extract_samples)
export(h_jags_get_data)
export(h_jags_get_model_inits)
export(h_jags_join_models)
export(h_jags_write_model)
export(h_model_dual_endpoint_beta)
export(h_model_dual_endpoint_rho)
export(h_model_dual_endpoint_sigma2betaw)
export(h_model_dual_endpoint_sigma2w)
export(h_next_best_eligible_doses)
export(h_next_best_mg_ci)
export(h_next_best_mg_doses_at_grid)
export(h_next_best_mg_plot)
export(h_next_best_mgsamples_plot)
export(h_next_best_ncrm_loss_plot)
export(h_next_best_td_plot)
export(h_next_best_tdsamples_plot)
export(h_null_if_na)
export(h_rapply)
export(h_slots)
export(h_test_named_numeric)
export(h_validate_combine_results)
export(is_logging_enabled)
export(log_trace)
export(logit)
export(match_within_tolerance)
export(maxDose)
export(maxSize)
export(mcmc)
export(minSize)
export(nextBest)
export(ngrid)
export(plotDualResponses)
export(plotGain)
export(prob)
export(probFunction)
export(probit)
export(saveSample)
export(set_seed)
export(size)
export(stopTrial)
export(test_format)
export(test_length)
export(test_probabilities)
export(test_probability)
export(test_probability_range)
export(test_range)
export(tidy)
export(windowLength)
exportClasses(Backfill)
exportClasses(CohortSize)
exportClasses(CohortSizeConst)
exportClasses(CohortSizeDLT)
exportClasses(CohortSizeMax)
exportClasses(CohortSizeMin)
exportClasses(CohortSizeOrdinal)
exportClasses(CohortSizeParts)
exportClasses(CohortSizeRandom)
exportClasses(CohortSizeRange)
exportClasses(CrmPackClass)
exportClasses(DADesign)
exportClasses(DALogisticLogNormal)
exportClasses(DASimulations)
exportClasses(Data)
exportClasses(DataDA)
exportClasses(DataDual)
exportClasses(DataGrouped)
exportClasses(DataMixture)
exportClasses(DataOrdinal)
exportClasses(DataParts)
exportClasses(Design)
exportClasses(DesignGrouped)
exportClasses(DesignOrdinal)
exportClasses(DualDesign)
exportClasses(DualEndpoint)
exportClasses(DualEndpointBeta)
exportClasses(DualEndpointEmax)
exportClasses(DualEndpointRW)
exportClasses(DualResponsesDesign)
exportClasses(DualResponsesSamplesDesign)
exportClasses(DualSimulations)
exportClasses(DualSimulationsSummary)
exportClasses(EffFlexi)
exportClasses(Effloglog)
exportClasses(FractionalCRM)
exportClasses(GeneralData)
exportClasses(GeneralModel)
exportClasses(GeneralSimulations)
exportClasses(GeneralSimulationsSummary)
exportClasses(Increments)
exportClasses(IncrementsDoseLevels)
exportClasses(IncrementsHSRBeta)
exportClasses(IncrementsMaxToxProb)
exportClasses(IncrementsMin)
exportClasses(IncrementsOrdinal)
exportClasses(IncrementsRelative)
exportClasses(IncrementsRelativeDLT)
exportClasses(IncrementsRelativeDLTCurrent)
exportClasses(IncrementsRelativeParts)
exportClasses(LogisticIndepBeta)
exportClasses(LogisticKadane)
exportClasses(LogisticKadaneBetaGamma)
exportClasses(LogisticLogNormal)
exportClasses(LogisticLogNormalGrouped)
exportClasses(LogisticLogNormalMixture)
exportClasses(LogisticLogNormalOrdinal)
exportClasses(LogisticLogNormalSub)
exportClasses(LogisticNormal)
exportClasses(LogisticNormalFixedMixture)
exportClasses(LogisticNormalMixture)
exportClasses(McmcOptions)
exportClasses(ModelEff)
exportClasses(ModelLogNormal)
exportClasses(ModelParamsNormal)
exportClasses(ModelPseudo)
exportClasses(ModelTox)
exportClasses(NextBest)
exportClasses(NextBestDualEndpoint)
exportClasses(NextBestEWOC)
exportClasses(NextBestInfTheory)
exportClasses(NextBestMTD)
exportClasses(NextBestMaxGain)
exportClasses(NextBestMaxGainSamples)
exportClasses(NextBestMinDist)
exportClasses(NextBestNCRM)
exportClasses(NextBestNCRMLoss)
exportClasses(NextBestOrdinal)
exportClasses(NextBestProbMTDLTE)
exportClasses(NextBestProbMTDMinDist)
exportClasses(NextBestTD)
exportClasses(NextBestTDsamples)
exportClasses(NextBestThreePlusThree)
exportClasses(OneParExpPrior)
exportClasses(OneParLogNormalPrior)
exportClasses(Opening)
exportClasses(OpeningAll)
exportClasses(OpeningAny)
exportClasses(OpeningList)
exportClasses(OpeningMinCohorts)
exportClasses(OpeningMinDose)
exportClasses(OpeningMinResponses)
exportClasses(OpeningNone)
exportClasses(ProbitLogNormal)
exportClasses(ProbitLogNormalRel)
exportClasses(PseudoDualFlexiSimulations)
exportClasses(PseudoDualSimulations)
exportClasses(PseudoDualSimulationsSummary)
exportClasses(PseudoSimulations)
exportClasses(PseudoSimulationsSummary)
exportClasses(Recruitment)
exportClasses(RecruitmentRatio)
exportClasses(RecruitmentUnlimited)
exportClasses(RuleDesign)
exportClasses(RuleDesignOrdinal)
exportClasses(SafetyWindow)
exportClasses(SafetyWindowConst)
exportClasses(SafetyWindowSize)
exportClasses(Samples)
exportClasses(Simulations)
exportClasses(SimulationsSummary)
exportClasses(Stopping)
exportClasses(StoppingAll)
exportClasses(StoppingAny)
exportClasses(StoppingCohortsNearDose)
exportClasses(StoppingExternal)
exportClasses(StoppingHighestDose)
exportClasses(StoppingList)
exportClasses(StoppingLowestDoseHSRBeta)
exportClasses(StoppingMTDCV)
exportClasses(StoppingMTDdistribution)
exportClasses(StoppingMaxGainCIRatio)
exportClasses(StoppingMinCohorts)
exportClasses(StoppingMinPatients)
exportClasses(StoppingMissingDose)
exportClasses(StoppingOrdinal)
exportClasses(StoppingPatientsNearDose)
exportClasses(StoppingSpecificDose)
exportClasses(StoppingTDCIRatio)
exportClasses(StoppingTargetBiomarker)
exportClasses(StoppingTargetProb)
exportClasses(TDDesign)
exportClasses(TDsamplesDesign)
exportClasses(TITELogisticLogNormal)
exportMethods("&")
exportMethods("[")
exportMethods("|")
exportMethods(biomarker)
exportMethods(dose)
exportMethods(doseFunction)
exportMethods(efficacy)
exportMethods(efficacyFunction)
exportMethods(gain)
exportMethods(get)
exportMethods(maxDose)
exportMethods(maxRecruits)
exportMethods(maxSize)
exportMethods(minSize)
exportMethods(names)
exportMethods(nextBest)
exportMethods(openCohort)
exportMethods(plot)
exportMethods(plotDualResponses)
exportMethods(plotGain)
exportMethods(prob)
exportMethods(probFunction)
exportMethods(show)
exportMethods(simulate)
exportMethods(size)
exportMethods(stopTrial)
exportMethods(summary)
exportMethods(tidy)
exportMethods(update)
exportMethods(windowLength)
import(checkmate)
import(ggplot2)
import(methods)
import(tibble)
importFrom(GenSA,GenSA)
importFrom(Rdpack,reprompt)
importFrom(futile.logger,FATAL)
importFrom(futile.logger,TRACE)
importFrom(futile.logger,flog.logger)
importFrom(futile.logger,flog.threshold)
importFrom(futile.logger,flog.trace)
importFrom(ggplot2,aes)
importFrom(ggplot2,geom_histogram)
importFrom(ggplot2,ggplot)
importFrom(ggplot2,xlab)
importFrom(ggplot2,xlim)
importFrom(ggplot2,ylab)
importFrom(graphics,hist)
importFrom(graphics,legend)
importFrom(graphics,lines)
importFrom(graphics,matlines)
importFrom(graphics,matplot)
importFrom(graphics,plot)
importFrom(grid,grid.draw)
importFrom(gridExtra,arrangeGrob)
importFrom(kableExtra,add_footnote)
importFrom(kableExtra,add_header_above)
importFrom(kableExtra,collapse_rows)
importFrom(kableExtra,column_spec)
importFrom(kableExtra,kable)
importFrom(kableExtra,kable_styling)
importFrom(kableExtra,kbl)
importFrom(knitr,knit_print)
importFrom(lifecycle,badge)
importFrom(magrittr,"%>%")
importFrom(mvtnorm,rmvnorm)
importFrom(parallel,detectCores)
importFrom(parallel,makeCluster)
importFrom(parallelly,availableCores)
importFrom(rjags,jags.model)
importFrom(rjags,jags.samples)
importFrom(rlang,":=")
importFrom(rlang,.data)
importFrom(stats,approxfun)
importFrom(stats,binomial)
importFrom(stats,coef)
importFrom(stats,cov2cor)
importFrom(stats,dbeta)
importFrom(stats,dgamma)
importFrom(stats,gaussian)
importFrom(stats,glm)
importFrom(stats,lm)
importFrom(stats,mad)
importFrom(stats,median)
importFrom(stats,model.matrix)
importFrom(stats,optim)
importFrom(stats,pbeta)
importFrom(stats,pgamma)
importFrom(stats,plogis)
importFrom(stats,pnorm)
importFrom(stats,qgamma)
importFrom(stats,qlogis)
importFrom(stats,qnorm)
importFrom(stats,quantile)
importFrom(stats,rbinom)
importFrom(stats,rgamma)
importFrom(stats,rnorm)
importFrom(stats,runif)
importFrom(stats,setNames)
importFrom(stats,step)
importFrom(stats,uniroot)
importFrom(stats,var)
importFrom(stats,vcov)
importFrom(tools,file_ext)
importFrom(utils,browseURL)
importFrom(utils,capture.output)
importFrom(utils,data)
importFrom(utils,head)
importFrom(utils,help)
importFrom(utils,tail)
